@@ -9,7 +9,7 @@ def dask_getJtJdiag(self, m, W=None):
     """
     Return the diagonal of JtJ
     """
-    if self.gtgdiag is None:
+    if getattr(self, "_gtgdiag", None) is None:
 
         # Need to check if multiplying weights makes sense
         if W is None:
@@ -17,9 +17,9 @@ def dask_getJtJdiag(self, m, W=None):
         else:
             W = self._scale * W.diagonal()
         w = da.from_array(W)[:, None]
-        self.gtgdiag = da.sum((w * self.getJ(m)) ** 2, axis=0).compute()
+        self._gtgdiag = da.sum((w * self.getJ(m)) ** 2, axis=0).compute()
 
-    return self.gtgdiag
+    return self._gtgdiag
 
 
 Sim.getJtJdiag = dask_getJtJdiag
