@@ -1,13 +1,11 @@
 import numpy as np
 import scipy.sparse as sp
 from scipy.constants import mu_0
-import properties
-from ...utils.code_utils import deprecate_class
 from discretize.utils import Zero
 
 from ... import props
 from ...data import Data
-from ...utils import mkvc
+from ...utils import mkvc, validate_type
 from ..base import BaseEMSimulation
 from ..utils import omega
 from .survey import Survey
@@ -57,18 +55,41 @@ class BaseFDEMSimulation(BaseEMSimulation):
 
     fieldsPair = FieldsFDEM
 
-    mu, muMap, muDeriv = props.Invertible("Magnetic Permeability (H/m)", default=mu_0)
+    def __init__(self, mesh, survey=None, forward_only=False, **kwargs):
+        super().__init__(mesh=mesh, survey=survey, **kwargs)
+        self.forward_only = forward_only
 
-    mui, muiMap, muiDeriv = props.Invertible("Inverse Magnetic Permeability (m/H)")
+    @property
+    def survey(self):
+        """The simulations survey.
 
-    props.Reciprocal(mu, mui)
+        Returns
+        -------
+        SimPEG.electromagnetics.frequency_domain.survey.Survey
+        """
+        if self._survey is None:
+            raise AttributeError("Simulation must have a survey set")
+        return self._survey
 
-    forward_only = properties.Boolean(
-        "If True, A-inverse not stored at each frequency in forward simulation",
-        default=False,
-    )
+    @survey.setter
+    def survey(self, value):
+        if value is not None:
+            value = validate_type("survey", value, Survey, cast=False)
+        self._survey = value
 
-    survey = properties.Instance("a survey object", Survey, required=True)
+    @property
+    def forward_only(self):
+        """If True, A-inverse not stored at each frequency in forward simulation.
+
+        Returns
+        -------
+        bool
+        """
+        return self._forward_only
+
+    @forward_only.setter
+    def forward_only(self, value):
+        self._forward_only = validate_type("forward_only", value, bool)
 
     # @profile
     def fields(self, m=None):
@@ -86,30 +107,20 @@ class BaseFDEMSimulation(BaseEMSimulation):
         try:
             self.Ainv
         except AttributeError:
-            if self.verbose:
-                print("num_frequencies =", self.survey.num_frequencies)
-            self.Ainv = [None for i in range(self.survey.num_frequencies)]
-
-        if self.Ainv[0] is not None:
-            for i in range(self.survey.num_frequencies):
-                self.Ainv[i].clean()
-
-            if self.verbose:
-                print("Cleaning Ainv")
+            self.Ainv = len(self.survey.frequencies) * [None]
 
         f = self.fieldsPair(self)
 
-        for nf, freq in enumerate(self.survey.frequencies):
+        for i_f, freq in enumerate(self.survey.frequencies):
             A = self.getA(freq)
             rhs = self.getRHS(freq)
-            self.Ainv[nf] = self.solver(A, **self.solver_opts)
-            u = self.Ainv[nf] * rhs
+            Ainv = self.solver(A, **self.solver_opts)
+            u = Ainv * rhs
+            if not self.forward_only:
+                self.Ainv[i_f] = Ainv
+
             Srcs = self.survey.get_sources_by_frequency(freq)
             f[Srcs, self._solutionType] = u
-            if self.forward_only:
-                if self.verbose:
-                    print("Fields simulated for frequency {}".format(nf))
-                self.Ainv[nf].clean()
         return f
 
     # @profile
@@ -130,8 +141,7 @@ class BaseFDEMSimulation(BaseEMSimulation):
 
         self.model = m
 
-        # Jv = Data(self.survey)
-        Jv = []
+        Jv = Data(self.survey)
 
         for nf, freq in enumerate(self.survey.frequencies):
             for src in self.survey.get_sources_by_frequency(freq):
@@ -139,12 +149,11 @@ class BaseFDEMSimulation(BaseEMSimulation):
                 dA_dm_v = self.getADeriv(freq, u_src, v, adjoint=False)
                 dRHS_dm_v = self.getRHSDeriv(freq, src, v)
                 du_dm_v = self.Ainv[nf] * (-dA_dm_v + dRHS_dm_v)
-
                 for rx in src.receiver_list:
-                    Jv.append(rx.evalDeriv(src, self.mesh, f, du_dm_v=du_dm_v, v=v))
-        return np.hstack(Jv)
+                    Jv[src, rx] = rx.evalDeriv(src, self.mesh, f, du_dm_v=du_dm_v, v=v)
+
+        return Jv.dobs
 
-    # @profile
     def Jtvec(self, m, v, f=None):
         """
         Sensitivity transpose times a vector
@@ -203,19 +212,25 @@ class BaseFDEMSimulation(BaseEMSimulation):
         :return: (s_m, s_e) (nE or nF, nSrc)
         """
         Srcs = self.survey.get_sources_by_frequency(freq)
+        n_fields = sum(src._fields_per_source for src in Srcs)
         if self._formulation == "EB":
-            s_m = np.zeros((self.mesh.nF, len(Srcs)), dtype=complex)
-            s_e = np.zeros((self.mesh.nE, len(Srcs)), dtype=complex)
+            s_m = np.zeros((self.mesh.nF, n_fields), dtype=complex, order="F")
+            s_e = np.zeros((self.mesh.nE, n_fields), dtype=complex, order="F")
         elif self._formulation == "HJ":
-            s_m = np.zeros((self.mesh.nE, len(Srcs)), dtype=complex)
-            s_e = np.zeros((self.mesh.nF, len(Srcs)), dtype=complex)
+            s_m = np.zeros((self.mesh.nE, n_fields), dtype=complex, order="F")
+            s_e = np.zeros((self.mesh.nF, n_fields), dtype=complex, order="F")
 
-        for i, src in enumerate(Srcs):
+        i = 0
+        for src in Srcs:
+            ii = i + src._fields_per_source
             smi, sei = src.eval(self)
-
-            s_m[:, i] = s_m[:, i] + smi
-            s_e[:, i] = s_e[:, i] + sei
-
+            if not isinstance(smi, Zero) and smi.ndim == 1:
+                smi = smi[:, None]
+            if not isinstance(sei, Zero) and sei.ndim == 1:
+                sei = sei[:, None]
+            s_m[:, i:ii] = s_m[:, i:ii] + smi
+            s_e[:, i:ii] = s_e[:, i:ii] + sei
+            i = ii
         return s_m, s_e
 
 
@@ -253,9 +268,6 @@ class Simulation3DElectricField(BaseFDEMSimulation):
     _formulation = "EB"
     fieldsPair = Fields3DElectricField
 
-    def __init__(self, mesh, **kwargs):
-        super(Simulation3DElectricField, self).__init__(mesh, **kwargs)
-
     def getA(self, freq):
         """
         System matrix
@@ -271,12 +283,9 @@ class Simulation3DElectricField(BaseFDEMSimulation):
 
         MfMui = self.MfMui
         MeSigma = self.MeSigma
-        C = self.mesh.edgeCurl
+        C = self.mesh.edge_curl
 
-        return C.T * MfMui * C + 1j * omega(freq) * MeSigma
-
-    # def getADeriv(self, freq, u, v, adjoint=False):
-    #     return
+        return C.T.tocsr() * MfMui * C + 1j * omega(freq) * MeSigma
 
     def getADeriv_sigma(self, freq, u, v, adjoint=False):
         """
@@ -311,7 +320,7 @@ class Simulation3DElectricField(BaseFDEMSimulation):
 
         """
 
-        C = self.mesh.edgeCurl
+        C = self.mesh.edge_curl
 
         if adjoint:
             return self.MfMuiDeriv(C * u).T * (C * v)
@@ -339,7 +348,7 @@ class Simulation3DElectricField(BaseFDEMSimulation):
         """
 
         s_m, s_e = self.getSourceTerm(freq)
-        C = self.mesh.edgeCurl
+        C = self.mesh.edge_curl
         MfMui = self.MfMui
 
         return C.T * (MfMui * s_m) - 1j * omega(freq) * s_e
@@ -351,7 +360,7 @@ class Simulation3DElectricField(BaseFDEMSimulation):
         includes calls to derivatives in the sources
         """
 
-        C = self.mesh.edgeCurl
+        C = self.mesh.edge_curl
         MfMui = self.MfMui
         s_m, s_e = self.getSourceTerm(freq)
         s_mDeriv, s_eDeriv = src.evalDeriv(self, adjoint=adjoint)
@@ -395,9 +404,6 @@ class Simulation3DMagneticFluxDensity(BaseFDEMSimulation):
     _formulation = "EB"
     fieldsPair = Fields3DMagneticFluxDensity
 
-    def __init__(self, mesh, **kwargs):
-        super(Simulation3DMagneticFluxDensity, self).__init__(mesh, **kwargs)
-
     def getA(self, freq):
         """
         System matrix
@@ -413,13 +419,13 @@ class Simulation3DMagneticFluxDensity(BaseFDEMSimulation):
 
         MfMui = self.MfMui
         MeSigmaI = self.MeSigmaI
-        C = self.mesh.edgeCurl
+        C = self.mesh.edge_curl
         iomega = 1j * omega(freq) * sp.eye(self.mesh.nF)
 
-        A = C * (MeSigmaI * (C.T * MfMui)) + iomega
+        A = C * (MeSigmaI * (C.T.tocsr() * MfMui)) + iomega
 
-        if self._makeASymmetric is True:
-            return MfMui.T * A
+        if self._makeASymmetric:
+            return MfMui.T.tocsr() * A
         return A
 
     def getADeriv_sigma(self, freq, u, v, adjoint=False):
@@ -443,7 +449,7 @@ class Simulation3DMagneticFluxDensity(BaseFDEMSimulation):
         """
 
         MfMui = self.MfMui
-        C = self.mesh.edgeCurl
+        C = self.mesh.edge_curl
         MeSigmaIDeriv = self.MeSigmaIDeriv
         vec = C.T * (MfMui * u)
 
@@ -460,7 +466,7 @@ class Simulation3DMagneticFluxDensity(BaseFDEMSimulation):
         MfMui = self.MfMui
         MfMuiDeriv = self.MfMuiDeriv(u)
         MeSigmaI = self.MeSigmaI
-        C = self.mesh.edgeCurl
+        C = self.mesh.edge_curl
 
         if adjoint:
             return MfMuiDeriv.T * (C * (MeSigmaI.T * (C.T * v)))
@@ -493,7 +499,7 @@ class Simulation3DMagneticFluxDensity(BaseFDEMSimulation):
         """
 
         s_m, s_e = self.getSourceTerm(freq)
-        C = self.mesh.edgeCurl
+        C = self.mesh.edge_curl
         MeSigmaI = self.MeSigmaI
 
         RHS = s_m + C * (MeSigmaI * s_e)
@@ -516,7 +522,7 @@ class Simulation3DMagneticFluxDensity(BaseFDEMSimulation):
         :return: product of rhs deriv with a vector
         """
 
-        C = self.mesh.edgeCurl
+        C = self.mesh.edge_curl
         s_m, s_e = src.eval(self)
         MfMui = self.MfMui
 
@@ -576,9 +582,6 @@ class Simulation3DCurrentDensity(BaseFDEMSimulation):
     _formulation = "HJ"
     fieldsPair = Fields3DCurrentDensity
 
-    def __init__(self, mesh, **kwargs):
-        super(Simulation3DCurrentDensity, self).__init__(mesh, **kwargs)
-
     def getA(self, freq):
         """
         System matrix
@@ -594,13 +597,13 @@ class Simulation3DCurrentDensity(BaseFDEMSimulation):
 
         MeMuI = self.MeMuI
         MfRho = self.MfRho
-        C = self.mesh.edgeCurl
+        C = self.mesh.edge_curl
         iomega = 1j * omega(freq) * sp.eye(self.mesh.nF)
 
-        A = C * MeMuI * C.T * MfRho + iomega
+        A = C * MeMuI * C.T.tocsr() * MfRho + iomega
 
         if self._makeASymmetric is True:
-            return MfRho.T * A
+            return MfRho.T.tocsr() * A
         return A
 
     def getADeriv_rho(self, freq, u, v, adjoint=False):
@@ -630,7 +633,7 @@ class Simulation3DCurrentDensity(BaseFDEMSimulation):
 
         MeMuI = self.MeMuI
         MfRho = self.MfRho
-        C = self.mesh.edgeCurl
+        C = self.mesh.edge_curl
 
         if adjoint:
             vec = C * (MeMuI.T * (C.T * v))
@@ -645,7 +648,7 @@ class Simulation3DCurrentDensity(BaseFDEMSimulation):
 
     def getADeriv_mu(self, freq, u, v, adjoint=False):
 
-        C = self.mesh.edgeCurl
+        C = self.mesh.edge_curl
         MfRho = self.MfRho
 
         MeMuIDeriv = self.MeMuIDeriv(C.T * (MfRho * u))
@@ -688,7 +691,7 @@ class Simulation3DCurrentDensity(BaseFDEMSimulation):
         """
 
         s_m, s_e = self.getSourceTerm(freq)
-        C = self.mesh.edgeCurl
+        C = self.mesh.edge_curl
         MeMuI = self.MeMuI
 
         RHS = C * (MeMuI * s_m) - 1j * omega(freq) * s_e
@@ -715,7 +718,7 @@ class Simulation3DCurrentDensity(BaseFDEMSimulation):
         #     MfRho = self.MfRho
         #     return MfRho.T*RHS
 
-        C = self.mesh.edgeCurl
+        C = self.mesh.edge_curl
         MeMuI = self.MeMuI
         MeMuIDeriv = self.MeMuIDeriv
         s_mDeriv, s_eDeriv = src.evalDeriv(self, adjoint=adjoint)
@@ -766,9 +769,6 @@ class Simulation3DMagneticField(BaseFDEMSimulation):
     _formulation = "HJ"
     fieldsPair = Fields3DMagneticField
 
-    def __init__(self, mesh, **kwargs):
-        super(Simulation3DMagneticField, self).__init__(mesh, **kwargs)
-
     def getA(self, freq):
         """
         System matrix
@@ -786,9 +786,9 @@ class Simulation3DMagneticField(BaseFDEMSimulation):
 
         MeMu = self.MeMu
         MfRho = self.MfRho
-        C = self.mesh.edgeCurl
+        C = self.mesh.edge_curl
 
-        return C.T * (MfRho * C) + 1j * omega(freq) * MeMu
+        return C.T.tocsr() * (MfRho * C) + 1j * omega(freq) * MeMu
 
     def getADeriv_rho(self, freq, u, v, adjoint=False):
         """
@@ -809,19 +809,11 @@ class Simulation3DMagneticField(BaseFDEMSimulation):
         :return: derivative of the system matrix times a vector (nP,) or
             adjoint (nD,)
         """
-
-        MeMu = self.MeMu
-        C = self.mesh.edgeCurl
+        C = self.mesh.edge_curl
         if adjoint:
             return self.MfRhoDeriv(C * u, C * v, adjoint)
         return C.T * self.MfRhoDeriv(C * u, v, adjoint)
 
-        # MfRhoDeriv = self.MfRhoDeriv(C*u)
-
-        # if adjoint:
-        #     return MfRhoDeriv.T * (C * v)
-        # return C.T * (MfRhoDeriv * v)
-
     def getADeriv_mu(self, freq, u, v, adjoint=False):
         MeMuDeriv = self.MeMuDeriv(u)
 
@@ -851,7 +843,7 @@ class Simulation3DMagneticField(BaseFDEMSimulation):
         """
 
         s_m, s_e = self.getSourceTerm(freq)
-        C = self.mesh.edgeCurl
+        C = self.mesh.edge_curl
         MfRho = self.MfRho
 
         return s_m + C.T * (MfRho * s_e)
@@ -869,7 +861,7 @@ class Simulation3DMagneticField(BaseFDEMSimulation):
         """
 
         _, s_e = src.eval(self)
-        C = self.mesh.edgeCurl
+        C = self.mesh.edge_curl
         MfRho = self.MfRho
 
         # MfRhoDeriv = self.MfRhoDeriv(s_e)
@@ -885,28 +877,3 @@ class Simulation3DMagneticField(BaseFDEMSimulation):
         s_mDeriv, s_eDeriv = src.evalDeriv(self, adjoint=adjoint)
 
         return RHSDeriv + s_mDeriv(v) + C.T * (MfRho * s_eDeriv(v))
-
-
-############
-# Deprecated
-############
-
-
-@deprecate_class(removal_version="0.16.0", error=True)
-class Problem3D_e(Simulation3DElectricField):
-    pass
-
-
-@deprecate_class(removal_version="0.16.0", error=True)
-class Problem3D_b(Simulation3DMagneticFluxDensity):
-    pass
-
-
-@deprecate_class(removal_version="0.16.0", error=True)
-class Problem3D_h(Simulation3DMagneticField):
-    pass
-
-
-@deprecate_class(removal_version="0.16.0", error=True)
-class Problem3D_j(Simulation3DCurrentDensity):
-    pass
