@@ -1,86 +1,33 @@
-import sys
 import numpy as np
 from numpy.lib import recfunctions as recFunc
 
 from ..frequency_domain.survey import Survey
 from ...data import Data as BaseData
 from ...utils import mkvc
-from .sources import Planewave_xy_1Dprimary, Planewave_xy_1DhomotD
-from .receivers import Point3DImpedance, Point3DTipper
+from .sources import PlanewaveXYPrimary
+from .receivers import PointNaturalSource, Point3DTipper
 from .utils.plot_utils import DataNSEMPlotMethods
 
-#########
-# Survey
-#########
-
-
-# class Survey(BaseSurvey):
-#     """
-#     Survey class for NSEM.
-
-#     **Requried**
-#     :param list source_list: List of sources associated with the survey
-
-
-#     **Optional**
-#     """
-#     srcPair = BaseNSEMSrc
-
-#     def __init__(self, source_list, **kwargs):
-#         # Sort these by frequency
-#         self.source_list = source_list
-#         BaseSurvey.__init__(self, **kwargs)
-
-#         _freqDict = {}
-#         for src in source_list:
-#             if src.frequency not in _freqDict:
-#                 _freqDict[src.frequency] = []
-#             _freqDict[src.frequency] += [src]
-
-#         self._freqDict = _freqDict
-#         self._freqs = sorted([f for f in self._freqDict])
-
-#     @property
-#     def freqs(self):
-#         """Frequencies"""
-#         return self._freqs
-
-#     @property
-#     def num_frequencies(self):
-#         """Number of frequencies"""
-#         return len(self._freqDict)
-
-#     def getSrcByFreq(self, freq):
-#         """Returns the sources associated with a specific frequency."""
-#         assert freq in self._freqDict, "The requested frequency is not in this survey."
-#         return self._freqDict[freq]
-
-#     def eval(self, f):
-#         """
-#         Evalute and return Data given calculated fields
-
-#         :param SimPEG.electromagnetics.frequency_domain.fields.FieldsFDEM f: A NSEM fileds object to evaluate data from
-#         :retype: SimPEG.EM.NSEM.Data
-#         :return: NSEM Data object
-#         """
-#         data = Data(self)
-#         for src in self.source_list:
-#             sys.stdout.flush()
-#             for rx in src.receiver_list:
-#                 data[src, rx] = rx.eval(src, self.mesh, f)
-#         return data
-
-#     def evalDeriv(self, f):
-#         raise Exception('Use Sources to project fields deriv.')
-
 #########
 # Data
 #########
 
 
 class Data(BaseData, DataNSEMPlotMethods):
-    """
-    Data class for NSEMdata. Stores the data vector indexed by the survey.
+    """Data class for NSEMdata.
+
+    Stores the data vector indexed by the survey.
+
+    Parameters
+    ----------
+    survey : SimPEG.survey.Survey
+        Natural source EM survey
+    dobs : numpy.ndarray
+        Observed data
+    relative_error : numpy.ndarray, optional
+        Relative error
+    noise_floor : numpy.ndarray, optional
+        Noise floor
     """
 
     def __init__(self, survey, dobs=None, relative_error=None, noise_floor=None):
@@ -90,10 +37,15 @@ class Data(BaseData, DataNSEMPlotMethods):
         """
         Returns a numpy.recarray for a SimpegNSEM impedance data object.
 
-        :param returnType: Switches between returning a rec array where the impedance is split to real and imaginary ('RealImag') or is a complex ('Complex')
-        :type returnType: str, optional
-        :rtype: numpy.recarray
-        :return: Record array with data, with indexed columns
+        Parameters
+        ----------
+        returnType : str, default: "RealImag:
+            Switches between returning a receiver array where the impedance is split to real and imaginary ('RealImag') or is a complex ('Complex')
+
+        Returns
+        -------
+        numpy.recarray
+            Record array with data, with indexed columns
         """
 
         # Define the record fields
@@ -184,18 +136,22 @@ class Data(BaseData, DataNSEMPlotMethods):
 
     @classmethod
     def fromRecArray(cls, recArray, srcType="primary"):
-        """
-        Class method that reads in a numpy record array to NSEMdata object.
+        """Class method that reads in a numpy record array to NSEMdata object.
 
-        :param recArray: Record array with the data. Has to have ('freq','x','y','z') columns and some ('zxx','zxy','zyx','zyy','tzx','tzy')
-        :type recArray: numpy.recarray
-
-        :param srcType: The type of SimPEG.EM.NSEM.SrcNSEM to be used
-        :type srcType: str, optional
+        Parameters
+        ----------
+        recArray : numpy.ndarray
+            Record array with the data. Has to have ('freq','x','y','z') columns and some ('zxx','zxy','zyx','zyy','tzx','tzy')
+        srcType : str, default: "primary"
+            The type of SimPEG.EM.NSEM.SrcNSEM to be used. Either "primary" or "total"
 
+        Returns
+        -------
+        SimPEG.electromagnetics.natural_source.sources.SrcNSEM
+            Natural source
         """
         if srcType == "primary":
-            src = Planewave_xy_1Dprimary
+            src = PlanewaveXYPrimary
         elif srcType == "total":
             src = Planewave_xy_1DhomotD
         else:
@@ -233,18 +189,18 @@ class Data(BaseData, DataNSEMPlotMethods):
                             dataList.append(dFreq[rxType][notNaNind].imag.copy())
                         elif "z" in rxType:
                             receiver_list.append(
-                                Point3DImpedance(locs, rxType[1:3], "real")
+                                PointNaturalSource(locs, rxType[1:3], "real")
                             )
                             dataList.append(dFreq[rxType][notNaNind].real.copy())
                             receiver_list.append(
-                                Point3DImpedance(locs, rxType[1:3], "imag")
+                                PointNaturalSource(locs, rxType[1:3], "imag")
                             )
                             dataList.append(dFreq[rxType][notNaNind].imag.copy())
                     else:
                         component = "real" if "r" in rxType else "imag"
                         if "z" in rxType:
                             receiver_list.append(
-                                Point3DImpedance(locs, rxType[1:3], component)
+                                PointNaturalSource(locs, rxType[1:3], component)
                             )
                             dataList.append(dFreq[rxType][notNaNind].copy())
                         if "t" in rxType:
