@@ -1,6 +1,3 @@
-from __future__ import print_function
-from __future__ import division
-
 from matplotlib import pyplot as plt, colors, numpy as np
 
 
