@@ -11,12 +11,12 @@ def get1DEfields(m1d, sigma, freq, sourceAmp=1.0):
     """Function to get 1D electrical fields"""
 
     # Get the gradient
-    G = m1d.nodalGrad
+    G = m1d.nodal_gradient
     # Mass matrices
     # Magnetic permeability
-    Mmu = sdiag(m1d.vol * (1.0 / mu_0))
+    Mmu = sdiag(m1d.cell_volumes * (1.0 / mu_0))
     # Conductivity
-    Msig = m1d.getFaceInnerProduct(sigma)
+    Msig = m1d.get_face_inner_product(sigma)
     # Set up the solution matrix
     A = G.T * Mmu * G + 1j * 2.0 * np.pi * freq * Msig
     # Define the inner part of the solution matrix
@@ -25,7 +25,7 @@ def get1DEfields(m1d, sigma, freq, sourceAmp=1.0):
     Aio = A[1:-1, [0, -1]]
 
     # Set the boundary conditions
-    Ed, Eu, Hd, Hu = getEHfields(m1d, sigma, freq, m1d.vectorNx)
+    Ed, Eu, Hd, Hu = getEHfields(m1d, sigma, freq, m1d.nodes_x)
     Etot = Ed + Eu
     if sourceAmp is not None:
         Etot = (
