@@ -1,44 +1,68 @@
 import numpy as np
 import scipy.sparse as sp
-import properties
-from ....utils.code_utils import deprecate_class
 
-from ....utils import mkvc, Zero
+from ....utils import mkvc, Zero, validate_type, validate_string
 from ....data import Data
-from ...base import BaseEMSimulation
+from ....base import BaseElectricalPDESimulation
 from .survey import Survey
 from .fields import Fields3DCellCentered, Fields3DNodal
 from .utils import _mini_pole_pole
 from discretize.utils import make_boundary_bool
 
 
-class BaseDCSimulation(BaseEMSimulation):
+class BaseDCSimulation(BaseElectricalPDESimulation):
     """
     Base DC Problem
     """
 
-    survey = properties.Instance("a DC survey object", Survey, required=True)
-
-    storeJ = properties.Bool("store the sensitivity matrix?", default=False)
-
     _mini_survey = None
 
     Ainv = None
-    _Jmatrix = None
-    gtgdiag = None
 
-    def __init__(self, *args, **kwargs):
-        miniaturize = kwargs.pop("miniaturize", False)
-        super().__init__(*args, **kwargs)
+    def __init__(self, mesh, survey=None, storeJ=False, miniaturize=False, **kwargs):
+        super().__init__(mesh=mesh, survey=survey, **kwargs)
+        self.storeJ = storeJ
         # Do stuff to simplify the forward and JTvec operation if number of dipole
         # sources is greater than the number of unique pole sources
+        miniaturize = validate_type("miniaturize", miniaturize, bool)
         if miniaturize:
             self._dipoles, self._invs, self._mini_survey = _mini_pole_pole(self.survey)
 
+    @property
+    def survey(self):
+        """The DC survey object.
+
+        Returns
+        -------
+        SimPEG.electromagnetics.static.resistivity.survey.Survey
+        """
+        if self._survey is None:
+            raise AttributeError("Simulation must have a survey")
+        return self._survey
+
+    @survey.setter
+    def survey(self, value):
+        if value is not None:
+            value = validate_type("survey", value, Survey, cast=False)
+        self._survey = value
+
+    @property
+    def storeJ(self):
+        """Whether to store the sensitivity matrix
+
+        Returns
+        -------
+        bool
+        """
+        return self._storeJ
+
+    @storeJ.setter
+    def storeJ(self, value):
+        self._storeJ = validate_type("storeJ", value, bool)
+
     def fields(self, m=None, calcJ=True):
         if m is not None:
             self.model = m
-            self._Jmatrix = None
 
         f = self.fieldsPair(self)
         if self.Ainv is not None:
@@ -52,7 +76,7 @@ class BaseDCSimulation(BaseEMSimulation):
         return f
 
     def getJ(self, m, f=None):
-        if self._Jmatrix is None:
+        if getattr(self, "_Jmatrix", None) is None:
             if f is None:
                 f = self.fields(m)
             self._Jmatrix = self._Jtvec(m, v=None, f=f).T
@@ -76,7 +100,7 @@ class BaseDCSimulation(BaseEMSimulation):
         """
         Return the diagonal of JtJ
         """
-        if self.gtgdiag is None:
+        if getattr(self, "_gtgdiag", None) is None:
             J = self.getJ(m)
 
             if W is None:
@@ -88,17 +112,18 @@ class BaseDCSimulation(BaseEMSimulation):
             for i in range(J.shape[0]):
                 diag += (W[i]) * (J[i] * J[i])
 
-            self.gtgdiag = diag
-        return self.gtgdiag
+            self._gtgdiag = diag
+        return self._gtgdiag
 
     def Jvec(self, m, v, f=None):
         """
         Compute sensitivity matrix (J) and vector (v) product.
         """
-
         if f is None:
             f = self.fields(m)
 
+        self.model = m
+
         if self.storeJ:
             J = self.getJ(m, f=f)
             return J.dot(v)
@@ -203,31 +228,29 @@ class BaseDCSimulation(BaseEMSimulation):
         :return: q (nC or nN, nSrc)
         """
 
-        if self._mini_survey is not None:
-            Srcs = self._mini_survey.source_list
-        else:
-            Srcs = self.survey.source_list
+        if getattr(self, "_q", None) is None:
+            if self._mini_survey is not None:
+                Srcs = self._mini_survey.source_list
+            else:
+                Srcs = self.survey.source_list
 
-        if self._formulation == "EB":
-            n = self.mesh.nN
+            if self._formulation == "EB":
+                n = self.mesh.nN
 
-        elif self._formulation == "HJ":
-            n = self.mesh.nC
+            elif self._formulation == "HJ":
+                n = self.mesh.nC
 
-        q = np.zeros((n, len(Srcs)), order="F")
+            q = np.zeros((n, len(Srcs)), order="F")
 
-        for i, source in enumerate(Srcs):
-            q[:, i] = source.eval(self)
-        return q
+            for i, source in enumerate(Srcs):
+                q[:, i] = source.eval(self)
+            self._q = q
+        return self._q
 
     @property
     def deleteTheseOnModelUpdate(self):
-        toDelete = super(BaseDCSimulation, self).deleteTheseOnModelUpdate
-        if self._Jmatrix is not None:
-            toDelete += ["_Jmatrix"]
-        if self.gtgdiag is not None:
-            toDelete += ["gtgdiag"]
-        return toDelete
+        toDelete = super().deleteTheseOnModelUpdate
+        return toDelete + ["_Jmatrix", "_gtgdiag"]
 
     def _mini_survey_data(self, d_mini):
         if self._mini_survey is not None:
@@ -263,18 +286,32 @@ class Simulation3DCellCentered(BaseDCSimulation):
     _formulation = "HJ"  # CC potentials means J is on faces
     fieldsPair = Fields3DCellCentered
 
-    bc_type = properties.StringChoice(
-        "Type of boundary condition to use for simulation. Note that Robin and Mixed "
-        "are equivalent.",
-        choices=["Dirichlet", "Neumann", "Robin", "Mixed"],
-        default="Robin",
-    )
+    def __init__(self, mesh, survey=None, bc_type="Robin", **kwargs):
 
-    def __init__(self, mesh, **kwargs):
-
-        BaseDCSimulation.__init__(self, mesh, **kwargs)
+        super().__init__(mesh=mesh, survey=survey, **kwargs)
+        self.bc_type = bc_type
         self.setBC()
 
+    @property
+    def bc_type(self):
+        """Type of boundary condition to use for simulation.
+
+        Returns
+        -------
+        {"Dirichlet", "Neumann", "Robin", "Mixed"}
+
+        Notes
+        -----
+        Robin and Mixed are equivalent.
+        """
+        return self._bc_type
+
+    @bc_type.setter
+    def bc_type(self, value):
+        self._bc_type = validate_string(
+            "bc_type", value, ["Dirichlet", "Neumann", ("Robin", "Mixed")]
+        )
+
     def getA(self, resistivity=None):
         """
         Make the A matrix for the cell centered DC resistivity problem
@@ -286,7 +323,7 @@ class Simulation3DCellCentered(BaseDCSimulation):
         if resistivity is None:
             MfRhoI = self.MfRhoI
         else:
-            MfRhoI = self.mesh.getFaceInnerProduct(resistivity, invMat=True)
+            MfRhoI = self.mesh.get_face_inner_product(resistivity, invert_matrix=True)
         A = D @ MfRhoI @ G
 
         if self.bc_type == "Neumann":
@@ -345,7 +382,7 @@ class Simulation3DCellCentered(BaseDCSimulation):
             return
         elif self.bc_type == "Neumann":
             alpha, beta, gamma = 0, 1, 0
-        else:  # self.bc_type == "Mixed":
+        else:
             boundary_faces = mesh.boundary_faces
             boundary_normals = mesh.boundary_face_outward_normals
             n_bf = len(boundary_faces)
@@ -401,23 +438,37 @@ class Simulation3DNodal(BaseDCSimulation):
     _formulation = "EB"  # N potentials means B is on faces
     fieldsPair = Fields3DNodal
 
-    bc_type = properties.StringChoice(
-        "Type of boundary condition to use for simulation. Note that Robin and Mixed "
-        "are equivalent.",
-        choices=["Neumann", "Robin", "Mixed"],
-        default="Robin",
-    )
-
-    def __init__(self, mesh, **kwargs):
-        super().__init__(mesh, **kwargs)
+    def __init__(self, mesh, survey=None, bc_type="Robin", **kwargs):
+        super().__init__(mesh=mesh, survey=survey, **kwargs)
         # Not sure why I need to do this
         # To evaluate mesh.aveE2CC, this is required....
         if mesh._meshType == "TREE":
-            mesh.nodalGrad
+            mesh.nodal_gradient
         elif mesh._meshType == "CYL":
-            self.bc_type == "Neumann"
+            bc_type == "Neumann"
+        self.bc_type = bc_type
         self.setBC()
 
+    @property
+    def bc_type(self):
+        """Type of boundary condition to use for simulation.
+
+        Returns
+        -------
+        {"Neumann", "Robin", "Mixed"}
+
+        Notes
+        -----
+        Robin and Mixed are equivalent.
+        """
+        return self._bc_type
+
+    @bc_type.setter
+    def bc_type(self, value):
+        self._bc_type = validate_string(
+            "bc_type", value, ["Neumann", ("Robin", "Mixed")]
+        )
+
     def getA(self, resistivity=None):
         """
         Make the A matrix for the cell centered DC resistivity problem
@@ -426,9 +477,9 @@ class Simulation3DNodal(BaseDCSimulation):
         if resistivity is None:
             MeSigma = self.MeSigma
         else:
-            MeSigma = self.mesh.getEdgeInnerProduct(1.0 / resistivity)
-        Grad = self.mesh.nodalGrad
-        A = Grad.T @ MeSigma @ Grad
+            MeSigma = self.mesh.get_edge_inner_product(1.0 / resistivity)
+        Grad = self.mesh.nodal_gradient
+        A = Grad.T.tocsr() @ MeSigma @ Grad
 
         if self.bc_type == "Neumann":
             # Handling Null space of A
@@ -440,7 +491,7 @@ class Simulation3DNodal(BaseDCSimulation):
             # Dirichlet BC type should already have failed
             # Also, this will fail if sigma is anisotropic
             try:
-                A = A + sp.diags(self._AvgBC @ self.sigma)
+                A = A + sp.diags(self._AvgBC @ self.sigma, format="csr")
             except ValueError as err:
                 if len(self.sigma) != len(self.mesh):
                     raise NotImplementedError(
@@ -457,7 +508,7 @@ class Simulation3DNodal(BaseDCSimulation):
         Product of the derivative of our system matrix with respect to the
         model and a vector
         """
-        Grad = self.mesh.nodalGrad
+        Grad = self.mesh.nodal_gradient
         if not adjoint:
             out = Grad.T @ self.MeSigmaDeriv(Grad @ u, v, adjoint)
         else:
@@ -567,18 +618,3 @@ class Simulation3DNodal(BaseDCSimulation):
 
 
 Simulation3DCellCentred = Simulation3DCellCentered  # UK and US!
-
-
-############
-# Deprecated
-############
-
-
-@deprecate_class(removal_version="0.16.0", error=True)
-class Problem3D_N(Simulation3DNodal):
-    pass
-
-
-@deprecate_class(removal_version="0.16.0", error=True)
-class Problem3D_CC(Simulation3DCellCentered):
-    pass
