@@ -1,9 +1,7 @@
 import numpy as np
-import properties
 
 from ....utils import mkvc
-from ...base import BaseEMSimulation
-from ....data import Data
+from ....simulation import BaseSimulation
 from .... import props
 
 from .survey import Survey
@@ -16,48 +14,143 @@ except ImportError:
     from empymod.transform import get_dlf_points
 from empymod.utils import check_hankel
 from ..utils import static_utils
+from ....utils import validate_type, validate_string
 
 
-class Simulation1DLayers(BaseEMSimulation):
+class Simulation1DLayers(BaseSimulation):
     """
     1D DC Simulation
     """
 
+    sigma, sigmaMap, sigmaDeriv = props.Invertible("Electrical conductivity (S/m)")
+    rho, rhoMap, rhoDeriv = props.Invertible("Electrical resistivity (Ohm m)")
+    props.Reciprocal(sigma, rho)
+
     thicknesses, thicknessesMap, thicknessesDeriv = props.Invertible(
         "thicknesses of the layers"
     )
 
-    survey = properties.Instance("a DC survey object", Survey, required=True)
-
-    storeJ = properties.Bool("store the sensitivity", default=False)
-
-    data_type = "volt"
-    hankel_pts_per_dec = None  # Default: Standard DLF
-
-    # TODO: using 51 filter coefficient could be overkill, use less if possible
-    hankel_filter = "key_51_2012"  # Default: Hankel filter
-
-    _Jmatrix = None
-    fix_Jmatrix = False
-
-    def __init__(self, **kwargs):
-        BaseEMSimulation.__init__(self, **kwargs)
+    def __init__(
+        self,
+        survey=None,
+        sigma=None,
+        sigmaMap=None,
+        rho=None,
+        rhoMap=None,
+        thicknesses=None,
+        thicknessesMap=None,
+        storeJ=False,
+        data_type="volt",
+        hankel_pts_per_dec=None,
+        hankel_filter="key_51_2012",
+        fix_Jmatrix=False,
+        **kwargs,
+    ):
+        super().__init__(survey=survey, **kwargs)
+        self.sigma = sigma
+        self.rho = rho
+        self.thicknesses = thicknesses
+        self.sigmaMap = sigmaMap
+        self.rhoMap = rhoMap
+        self.thicknessesMap = thicknessesMap
+        self.storeJ = storeJ
+        self.data_type = data_type
+        self.fix_Jmatrix = fix_Jmatrix
         try:
-            ht, htarg = check_hankel(
-                "fht", [self.hankel_filter, self.hankel_pts_per_dec], 1
-            )
-            self.fhtfilt = htarg[0]  # Store filter
-            self.hankel_pts_per_dec = htarg[1]  # Store pts_per_dec
+            ht, htarg = check_hankel("fht", [hankel_filter, hankel_pts_per_dec], 1)
+            self._fhtfilt = htarg[0]  # Store filter
+            self._hankel_pts_per_dec = htarg[1]  # Store pts_per_dec
         except ValueError:
             arg = {}
-            arg["dlf"] = self.hankel_filter
-            if self.hankel_pts_per_dec is not None:
-                arg["pts_per_dec"] = self.hankel_pts_per_dec
+            arg["dlf"] = hankel_filter
+            if hankel_pts_per_dec is not None:
+                arg["pts_per_dec"] = hankel_pts_per_dec
             ht, htarg = check_hankel("dlf", arg, 1)
-            self.fhtfilt = htarg["dlf"]  # Store filter
-            self.hankel_pts_per_dec = htarg["pts_per_dec"]  # Store pts_per_dec
-        self.hankel_filter = self.fhtfilt.name  # Store name
-        self.n_filter = self.fhtfilt.base.size
+            self._fhtfilt = htarg["dlf"]  # Store filter
+            self._hankel_pts_per_dec = htarg["pts_per_dec"]  # Store pts_per_dec
+        self._hankel_filter = self._fhtfilt.name
+
+    @property
+    def survey(self):
+        """The DC survey object.
+
+        Returns
+        -------
+        SimPEG.electromagnetics.static.resistivity.survey.Survey
+        """
+        if self._survey is None:
+            raise AttributeError("Simulation must have a survey.")
+        return self._survey
+
+    @survey.setter
+    def survey(self, value):
+        if value is not None:
+            value = validate_type("survey", value, Survey, cast=False)
+        self._survey = value
+
+    @property
+    def storeJ(self):
+        """Whether to store the sensitivity matrix.
+
+        Returns
+        -------
+        bool
+        """
+        return self._storeJ
+
+    @storeJ.setter
+    def storeJ(self, value):
+        self._storeJ = validate_type("storeJ", value, bool)
+
+    @property
+    def hankel_filter(self):
+        """The hankel filter key.
+
+        Returns
+        -------
+        str
+        """
+        return self._hankel_filter
+
+    @property
+    def hankel_pts_per_dec(self):
+        """Number of hankel transform points per decade.
+
+        Returns
+        -------
+        int
+        """
+        return self._hankel_pts_per_dec
+
+    @property
+    def fix_Jmatrix(self):
+        """Whether to fix the sensitivity matrix between iterations.
+
+        Returns
+        -------
+        bool
+        """
+        return self._fix_Jmatrix
+
+    @fix_Jmatrix.setter
+    def fix_Jmatrix(self, value):
+        self._fix_Jmatrix = validate_type("fix_Jmatrix", value, bool)
+
+    @property
+    def data_type(self):
+        """The type of data observered by the receivers.
+
+        Returns
+        -------
+        {"volt", "apparent_resistivity"}
+        """
+        return self._data_type
+
+    @data_type.setter
+    def data_type(self, value):
+        self._data_type = validate_string(
+            "data_type", value, ["volt", "apparent_resistivity"]
+        )
 
     def fields(self, m):
 
@@ -78,25 +171,31 @@ class Simulation1DLayers(BaseEMSimulation):
             T1 = T0
         PJ = (T0, None, None)
         try:
-            voltage = dlf(
-                PJ,
-                self.lambd,
-                self.offset,
-                self.fhtfilt,
-                self.hankel_pts_per_dec,
-                factAng=None,
-                ab=33,
-            ).real / (2 * np.pi)
+            voltage = (
+                dlf(
+                    PJ,
+                    self.lambd,
+                    self.offset,
+                    self._fhtfilt,
+                    self.hankel_pts_per_dec,
+                    factAng=None,
+                    ab=33,
+                ).real
+                / (2 * np.pi)
+            )
         except TypeError:
-            voltage = dlf(
-                PJ,
-                self.lambd,
-                self.offset,
-                self.fhtfilt,
-                self.hankel_pts_per_dec,
-                ang_fact=None,
-                ab=33,
-            ).real / (2 * np.pi)
+            voltage = (
+                dlf(
+                    PJ,
+                    self.lambd,
+                    self.offset,
+                    self._fhtfilt,
+                    self.hankel_pts_per_dec,
+                    ang_fact=None,
+                    ab=33,
+                ).real
+                / (2 * np.pi)
+            )
 
         # Assume dipole-dipole
         V = voltage.reshape((self.survey.nD, 4), order="F")
@@ -129,9 +228,7 @@ class Simulation1DLayers(BaseEMSimulation):
         """
         Generate Full sensitivity matrix using central difference
         """
-        if self._Jmatrix is not None:
-            return self._Jmatrix
-        else:
+        if getattr(self, "_Jmatrix", None) is None:
             if self.verbose:
                 print("Calculating J and storing")
             self.model = m
@@ -174,13 +271,10 @@ class Simulation1DLayers(BaseEMSimulation):
 
     @property
     def deleteTheseOnModelUpdate(self):
-        toDelete = super(Simulation1DLayers, self).deleteTheseOnModelUpdate
+        toDelete = super().deleteTheseOnModelUpdate
         if self.fix_Jmatrix:
             return toDelete
-
-        if self._Jmatrix is not None:
-            toDelete += ["_Jmatrix"]
-        return toDelete
+        return toDelete + ["_Jmatrix"]
 
     @property
     def electrode_separations(self):
@@ -217,10 +311,10 @@ class Simulation1DLayers(BaseEMSimulation):
         # TODO: only works isotropic sigma
         if getattr(self, "_lambd", None) is None:
             self._lambd = np.empty(
-                [self.offset.size, self.n_filter], order="F", dtype=complex
+                [self.offset.size, self._fhtfilt.base.size], order="F", dtype=complex
             )
             self.lambd[:, :], _ = get_dlf_points(
-                self.fhtfilt, self.offset, self.hankel_pts_per_dec
+                self._fhtfilt, self.offset, self.hankel_pts_per_dec
             )
         return self._lambd
 
@@ -231,7 +325,7 @@ class Simulation1DLayers(BaseEMSimulation):
     #     """
     #     # TODO: only works isotropic sigma
     #     if getattr(self, '_t', None) is None:
-    #         self._t = self.mesh.hx[:-1]
+    #         self._t = self.mesh.h[0][:-1]
     #     return self._t
 
     @property
