@@ -1,14 +1,7 @@
-from __future__ import absolute_import
-from __future__ import division
-from __future__ import print_function
-from __future__ import unicode_literals
-
 import numpy as np
-from scipy.interpolate import interp1d, NearestNDInterpolator
-import properties
-from ....utils.code_utils import deprecate_class, deprecate_property
 
-from ....utils import uniqueRows
+from ....utils.code_utils import deprecate_property, validate_string
+
 from ....survey import BaseSurvey
 from ..utils import drapeTopotoLoc
 from . import receivers as Rx
@@ -16,35 +9,70 @@ from . import sources as Src
 from ..utils import static_utils
 from SimPEG import data
 
-import warnings
-
 
 class Survey(BaseSurvey):
-    """
-    Base DC survey
+    """DC/IP survey class
+
+    Parameters
+    ----------
+    source_list : list of SimPEG.electromagnetic.static.resistivity.sources.BaseSrc
+        List of SimPEG DC/IP sources
+    survey_geometry : {"surface", "borehole", "general"}
+        Survey geometry.
+    survey_type : {"dipole-dipole", "pole-dipole", "dipole-pole", "pole-pole"}
+        Survey type.
     """
 
-    source_list = properties.List(
-        "A list of sources for the survey",
-        properties.Instance("A DC source", Src.BaseSrc),
-        default=[],
-    )
-
-    # Survey
-    survey_geometry = properties.StringChoice(
-        "Survey geometry of DC surveys",
-        default="surface",
-        choices=["surface", "borehole", "general"],
-    )
-
-    survey_type = properties.StringChoice(
-        "DC-IP Survey type",
-        default="dipole-dipole",
-        choices=["dipole-dipole", "pole-dipole", "dipole-pole", "pole-pole"],
-    )
-
-    def __init__(self, source_list, **kwargs):
+    def __init__(
+        self,
+        source_list,
+        survey_geometry="surface",
+        survey_type="dipole-dipole",
+        **kwargs,
+    ):
         super(Survey, self).__init__(source_list, **kwargs)
+        self.survey_geometry = survey_geometry
+        self.survey_type = survey_type
+
+    @property
+    def survey_geometry(self):
+        """Survey geometry
+
+        This property is deprecated.
+
+        Returns
+        -------
+        str
+            Survey geometry; one of {"surface", "borehole", "general"}
+        """
+        return self._survey_geometry
+
+    @survey_geometry.setter
+    def survey_geometry(self, var):
+        var = validate_string(
+            "survey_geometry", var, ("surface", "borehole", "general")
+        )
+        self._survey_geometry = var
+
+    @property
+    def survey_type(self):
+        """Survey type; one of {"dipole-dipole", "pole-dipole", "dipole-pole", "pole-pole"}
+
+        Returns
+        -------
+        str
+            Survey type; one of {"dipole-dipole", "pole-dipole", "dipole-pole", "pole-pole"}
+        """
+        return self._survey_type
+
+    @survey_type.setter
+    def survey_type(self, var):
+        var = validate_string(
+            "survey_type",
+            var,
+            ("dipole-dipole", "pole-dipole", "dipole-pole", "pole-pole"),
+        )
+        self._survey_type = var
 
     def __repr__(self):
         return (
@@ -55,7 +83,12 @@ class Survey(BaseSurvey):
     @property
     def locations_a(self):
         """
-        Location of the positive (+) current electrodes for each datum
+        Locations of the positive (+) current electrodes in the survey
+
+        Returns
+        -------
+        (nD, dim) numpy.ndarray
+            Locations of the positive (+) current electrodes in the survey
         """
         if getattr(self, "_locations_a", None) is None:
             self._set_abmn_locations()
@@ -64,7 +97,12 @@ class Survey(BaseSurvey):
     @property
     def locations_b(self):
         """
-        Location of the negative (-) current electrodes for each datum
+        Locations of the negative (-) current electrodes in the survey
+
+        Returns
+        -------
+        (nD, dim) numpy.ndarray
+            Locations of the negative (-) current electrodes in the survey
         """
         if getattr(self, "_locations_b", None) is None:
             self._set_abmn_locations()
@@ -73,7 +111,12 @@ class Survey(BaseSurvey):
     @property
     def locations_m(self):
         """
-        Location of the positive (+) potential electrodes for each datum
+        Locations of the positive (+) potential electrodes in the survey
+
+        Returns
+        -------
+        (nD, dim) numpy.ndarray
+            Locations of the positive (+) potential electrodes in the survey
         """
         if getattr(self, "_locations_m", None) is None:
             self._set_abmn_locations()
@@ -82,45 +125,26 @@ class Survey(BaseSurvey):
     @property
     def locations_n(self):
         """
-        Location of the negative (-) potential electrodes for each datum
+        Locations of the negative (-) potential electrodes in the survey
+
+        Returns
+        -------
+        (nD, dim) numpy.ndarray
+            Locations of the negative (-) potential electrodes in the survey
         """
         if getattr(self, "_locations_n", None) is None:
             self._set_abmn_locations()
         return self._locations_n
 
-    a_locations = deprecate_property(
-        locations_a,
-        "a_locations",
-        new_name="locations_a",
-        removal_version="0.16.0",
-        error=True,
-    )
-    b_locations = deprecate_property(
-        locations_b,
-        "b_locations",
-        new_name="locations_b",
-        removal_version="0.16.0",
-        error=True,
-    )
-    m_locations = deprecate_property(
-        locations_m,
-        "m_locations",
-        new_name="locations_m",
-        removal_version="0.16.0",
-        error=True,
-    )
-    n_locations = deprecate_property(
-        locations_n,
-        "n_locations",
-        new_name="locations_n",
-        removal_version="0.16.0",
-        error=True,
-    )
-
     @property
     def unique_electrode_locations(self):
         """
         Unique locations of the A, B, M, N electrodes
+
+        Returns
+        -------
+        (n, dim) numpy.ndarray
+            The set of unique electrode locations used in the survey
         """
         loc_a = self.locations_a
         loc_b = self.locations_b
@@ -132,7 +156,8 @@ class Survey(BaseSurvey):
         unique_electrode_locations,
         "electrode_locations",
         new_name="unique_electrode_locations",
-        removal_version="0.16.0",
+        removal_version="0.17.0",
+        error=True,
     )
 
     @property
@@ -140,12 +165,10 @@ class Survey(BaseSurvey):
         """
         Returns, in order, the source locations for all sources in the survey.
 
-        Input:
-        :param self: SimPEG.electromagnetics.static.resistivity.Survey
-
-        Output:
-        :return source_locations: List of np.ndarray containing the A and B
-        electrode locations.
+        Returns
+        -------
+        list of numpy.ndarray
+            List of length 2 containing the A and B electrode locations, in order.
         """
         src_a = []
         src_b = []
@@ -158,8 +181,28 @@ class Survey(BaseSurvey):
         return [np.vstack(src_a), np.vstack(src_b)]
 
     def set_geometric_factor(
-        self, space_type="half-space", data_type=None, survey_type=None,
+        self,
+        space_type="halfspace",
+        data_type=None,
+        survey_type=None,
     ):
+        """
+        Set and return the geometric factor for all data
+
+        Parameters
+        ----------
+        space_type : {'halfspace', 'wholespace'}
+            Calculate geometric factors using a half-space or whole-space formula.
+        data_type : str, default = ``None``
+            This input argument is now deprecated
+        survey_type : str, default = ``None``
+            This input argument is now deprecated
+
+        Returns
+        -------
+        (nD) numpy.ndarray
+            The geometric factor for each datum
+        """
         if data_type is not None:
             raise TypeError(
                 "The data_type kwarg has been removed, please set the data_type on the "
@@ -203,6 +246,10 @@ class Survey(BaseSurvey):
                     locations_b.append(
                         source.location[1].reshape([1, -1]).repeat(nRx, axis=0)
                     )
+                elif isinstance(source, Src.Multipole):
+                    location_tiled = np.tile(source.location, (nRx, 1))
+                    locations_a.append(location_tiled)
+                    locations_b.append(location_tiled)
                 # Pole RX
                 if isinstance(rx, Rx.Pole):
                     locations_m.append(rx.locations)
@@ -219,6 +266,7 @@ class Survey(BaseSurvey):
         self._locations_n = np.vstack(locations_n)
 
     def getABMN_locations(self):
+        """The 'getABMN_locations' method has been removed."""
         raise TypeError(
             "The getABMN_locations method has been Removed. Please instead "
             "ask for the property of interest: survey.locations_a, "
@@ -226,14 +274,29 @@ class Survey(BaseSurvey):
         )
 
     def drape_electrodes_on_topography(
-        self, mesh, actind, option="top", topography=None, force=False
+        self, mesh, ind_active, option="top", topography=None, force=False
     ):
-        """Shift electrode locations to be on [top] of the active cells."""
+        """Shift electrode locations to discrete surface topography.
+
+        Parameters
+        ----------
+        mesh : discretize.TensorMesh or discretize.TreeMesh
+            The mesh on which the discretized fields are computed
+        ind_active : numpy.ndarray of int or bool
+            Active topography cells
+        option :{"top", "center"}
+            Define topography at tops of cells or cell centers.
+        topography : (n, dim) numpy.ndarray, default = ``None``
+            Surface topography
+        force : bool, default = ``False``
+            If ``True`` force electrodes to surface even if borehole
+
+        """
         if self.survey_geometry == "surface":
-            loc_a = self.locations_a
-            loc_b = self.locations_b
-            loc_m = self.locations_m
-            loc_n = self.locations_n
+            loc_a = self.locations_a[:, :2]
+            loc_b = self.locations_b[:, :2]
+            loc_m = self.locations_m[:, :2]
+            loc_n = self.locations_n[:, :2]
             unique_electrodes, inv = np.unique(
                 np.vstack((loc_a, loc_b, loc_m, loc_n)), return_inverse=True, axis=0
             )
@@ -242,7 +305,7 @@ class Survey(BaseSurvey):
             inv_m, inv_n = inv[: len(loc_m)], inv[len(loc_m) :]
 
             electrodes_shifted = drapeTopotoLoc(
-                mesh, unique_electrodes, actind=actind, option=option
+                mesh, unique_electrodes, ind_active=ind_active, option=option
             )
             a_shifted = electrodes_shifted[inv_a]
             b_shifted = electrodes_shifted[inv_b]
@@ -252,8 +315,8 @@ class Survey(BaseSurvey):
             ind = 0
             for src in self.source_list:
                 a_loc, b_loc = a_shifted[ind], b_shifted[ind]
-                if isinstance(src, Src.Pole):
-                    src.location = a_loc
+                if isinstance(src, (Src.Pole, Src.Multipole)):
+                    src.location = [a_loc]
                 else:
                     src.location = [a_loc, b_loc]
                 for rx in src.receiver_list:
@@ -272,20 +335,13 @@ class Survey(BaseSurvey):
         elif self.survey_geometry == "borehole":
             raise Exception("Not implemented yet for borehole survey_geometry")
         else:
-            raise Exception("Input valid survey survey_geometry: surface or borehole")
+            raise Exception(
+                f"Input valid survey survey_geometry: {self.survey_geometry}"
+            )
 
     def drapeTopo(self, *args, **kwargs):
+        """This method is deprecated. See :meth:`drape_electrodes_on_topography`"""
         raise TypeError(
             "The drapeTopo method has been removed. Please instead "
             "use the drape_electrodes_on_topography method."
         )
-
-
-############
-# Deprecated
-############
-
-
-@deprecate_class(removal_version="0.16.0", error=True)
-class Survey_ky(Survey):
-    pass
