@@ -1,12 +1,8 @@
-from __future__ import division
 import numpy as np
-import scipy.sparse as sp
 from scipy.constants import epsilon_0
-from ...utils.code_utils import deprecate_class
 
 from ...fields import TimeFields
 from ...utils import mkvc, sdiag, Zero
-from ..utils import omega
 
 
 class FieldsTDEM(TimeFields):
@@ -139,7 +135,7 @@ class Fields3DMagneticFluxDensity(FieldsTDEM):
         self._MeSigmaI = self.simulation.MeSigmaI
         self._MeSigmaDeriv = self.simulation.MeSigmaDeriv
         self._MeSigmaIDeriv = self.simulation.MeSigmaIDeriv
-        self._edgeCurl = self.simulation.mesh.edgeCurl
+        self._edgeCurl = self.simulation.mesh.edge_curl
         self._MfMui = self.simulation.MfMui
         self._timeMesh = self.simulation.time_mesh
 
@@ -156,7 +152,7 @@ class Fields3DMagneticFluxDensity(FieldsTDEM):
         return Zero()
 
     def _dbdt(self, bSolution, source_list, tInd):
-        # self._timeMesh.faceDiv
+        # self._timeMesh.face_divergence
         dbdt = -self._edgeCurl * self._e(bSolution, source_list, tInd)
         for i, src in enumerate(source_list):
             s_m = src.s_m(self.simulation, self._times[tInd])
@@ -296,7 +292,7 @@ class Fields3DElectricField(FieldsTDEM):
         self._MeSigmaI = self.simulation.MeSigmaI
         self._MeSigmaDeriv = self.simulation.MeSigmaDeriv
         self._MeSigmaIDeriv = self.simulation.MeSigmaIDeriv
-        self._edgeCurl = self.simulation.mesh.edgeCurl
+        self._edgeCurl = self.simulation.mesh.edge_curl
         self._MfMui = self.simulation.MfMui
 
     def _TLoc(self, fieldType):
@@ -337,7 +333,7 @@ class Fields3DElectricField(FieldsTDEM):
             "To obtain b-fields, please use Simulation3DMagneticFluxDensity"
         )
         # dbdt = self._dbdt(eSolution, source_list, tInd)
-        # dt = self.simulation.time_mesh.hx
+        # dt = self.simulation.time_mesh.h[0]
         # # assume widths of "ghost cells" same on either end
         # dtn = np.hstack([dt[0], 0.5*(dt[1:] + dt[:-1]), dt[-1]])
         # return dtn[tInd] * dbdt
@@ -412,7 +408,7 @@ class Fields3DMagneticField(FieldsTDEM):
 
     def startup(self):
         self._times = self.simulation.times
-        self._edgeCurl = self.simulation.mesh.edgeCurl
+        self._edgeCurl = self.simulation.mesh.edge_curl
         self._MeMuI = self.simulation.MeMuI
         self._MeMu = self.simulation.MeMu
         self._MfRho = self.simulation.MfRho
@@ -553,11 +549,14 @@ class Fields3DMagneticField(FieldsTDEM):
         )
 
     def _charge(self, hSolution, source_list, tInd):
-        vol = sdiag(self.simulation.mesh.vol)
+        vol = sdiag(self.simulation.mesh.cell_volumes)
         return (
             epsilon_0
             * vol
-            * (self.simulation.mesh.faceDiv * self._e(hSolution, source_list, tInd))
+            * (
+                self.simulation.mesh.face_divergence
+                * self._e(hSolution, source_list, tInd)
+            )
         )
 
 
@@ -576,7 +575,7 @@ class Fields3DCurrentDensity(FieldsTDEM):
 
     def startup(self):
         self._times = self.simulation.times
-        self._edgeCurl = self.simulation.mesh.edgeCurl
+        self._edgeCurl = self.simulation.mesh.edge_curl
         self._MeMuI = self.simulation.MeMuI
         self._MfRho = self.simulation.MfRho
         self._MfRhoDeriv = self.simulation.MfRhoDeriv
@@ -645,12 +644,12 @@ class Fields3DCurrentDensity(FieldsTDEM):
         return self.simulation.MfI * self._MfRhoDeriv(jSolution, v)
 
     def _charge(self, jSolution, source_list, tInd):
-        vol = sdiag(self.simulation.mesh.vol)
+        vol = sdiag(self.simulation.mesh.cell_volumes)
         return vol * self._charge_density(jSolution, source_list, tInd)
 
     def _charge_density(self, jSolution, source_list, tInd):
         return epsilon_0 * (
-            self.simulation.mesh.faceDiv * self._e(jSolution, source_list, tInd)
+            self.simulation.mesh.face_divergence * self._e(jSolution, source_list, tInd)
         )
 
     def _dbdt(self, jSolution, source_list, tInd):
@@ -678,36 +677,3 @@ class Fields3DCurrentDensity(FieldsTDEM):
         return self.simulation.MeI * (
             self.simulation.MeMu * self._dhdtDeriv_m(tInd, src, v)
         )
-
-
-############
-# Deprecated
-############
-@deprecate_class(removal_version="0.16.0", error=True)
-class Fields_Derivs_eb(FieldsDerivativesEB):
-    pass
-
-
-@deprecate_class(removal_version="0.16.0", error=True)
-class Fields_Derivs_hj(FieldsDerivativesHJ):
-    pass
-
-
-@deprecate_class(removal_version="0.16.0", error=True)
-class Fields3D_b(Fields3DMagneticFluxDensity):
-    pass
-
-
-@deprecate_class(removal_version="0.16.0", error=True)
-class Fields3D_e(Fields3DElectricField):
-    pass
-
-
-@deprecate_class(removal_version="0.16.0", error=True)
-class Fields3D_h(Fields3DMagneticField):
-    pass
-
-
-@deprecate_class(removal_version="0.16.0", error=True)
-class Fields3D_j(Fields3DCurrentDensity):
-    pass
