@@ -1,12 +1,16 @@
+import discretize
 import numpy as np
 import scipy.sparse as sp
-import properties
-from ...utils.code_utils import deprecate_class, deprecate_property
 
 from ...simulation import BaseSimulation
 from ... import props
 from ... import maps
-from ...utils import mkvc
+from ...utils import (
+    mkvc,
+    validate_type,
+    validate_ndarray_with_shape,
+    validate_active_indices,
+)
 
 from .survey import SurveyVRM
 from .receivers import Point, SquareLoop
@@ -21,102 +25,101 @@ class BaseVRMSimulation(BaseSimulation):
     """"""
 
     _AisSet = False
-    refinement_factor = properties.Integer("Sensitivity refinement factor", min=0)
-    refinement_distance = properties.Array(
-        "Sensitivity refinement radii from sources", dtype=float
-    )
-    indActive = properties.Array("Topography active cells", dtype=bool)
 
-    ref_factor = deprecate_property(
-        refinement_factor,
-        "ref_factor",
-        new_name="refinement_factor",
-        removal_version="0.16.0",
-        error=True,
-    )
-    ref_radius = deprecate_property(
-        refinement_distance,
-        "ref_radius",
-        new_name="refinement_distance",
-        removal_version="0.16.0",
-        error=True,
-    )
+    def __init__(
+        self,
+        mesh,
+        survey=None,
+        refinement_factor=None,
+        refinement_distance=None,
+        indActive=None,
+        **kwargs,
+    ):
+        super().__init__(mesh=mesh, survey=survey, **kwargs)
 
-    def __init__(self, mesh=None, **kwargs):
-
-        refinement_factor = kwargs.pop("refinement_factor", None)
-        refinement_distance = kwargs.pop("refinement_distance", None)
-        indActive = kwargs.pop("indActive", None)
-
-        if len(mesh.h) != 3:
-            raise ValueError(
-                "Mesh must be 3D tensor or 3D tree. Current mesh is {}".format(
-                    len(mesh.h)
-                )
-            )
-
-        super(BaseVRMSimulation, self).__init__(mesh, **kwargs)
-
-        if refinement_factor is None and refinement_distance is None:
-            self.refinement_factor = 3
-            self.refinement_distance = list(
-                1.25
-                * np.mean(
-                    np.r_[np.min(mesh.h[0]), np.min(mesh.h[1]), np.min(mesh.h[2])]
-                )
-                * np.arange(1, 4)
-            )
-        elif refinement_factor is None and refinement_distance is not None:
-            self.refinement_factor = len(refinement_distance)
-            self.refinement_distance = refinement_distance
-        elif refinement_factor is not None and refinement_distance is None:
-            self.refinement_factor = refinement_factor
-            self.refinement_distance = list(
+        if refinement_distance is None:
+            if refinement_factor is None:
+                refinement_factor = 3
+            refinement_distance = (
                 1.25
                 * np.mean(
                     np.r_[np.min(mesh.h[0]), np.min(mesh.h[1]), np.min(mesh.h[2])]
                 )
                 * np.arange(1, refinement_factor + 1)
             )
-        else:
-            self.refinement_factor = refinement_factor
-            self.refinement_distance = refinement_distance
-
+        self.refinement_distance = refinement_distance
         if indActive is None:
-            self.indActive = np.ones(mesh.nC, dtype=bool)
-        else:
-            self.indActive = indActive
+            indActive = np.ones(self.mesh.n_cells, dtype=bool)
+        self.indActive = indActive
 
-    @properties.observer("refinement_factor")
-    def _refinement_factor_observer(self, change):
-        if change["value"] > 4:
-            print(
-                "Refinement factor larger than 4 may result in computations which exceed memory limits"
-            )
-        if self.refinement_distance is not None and change["value"] != len(
-            self.refinement_distance
-        ):
-            print(
-                "Number of refinement radii currently DOES NOT match refinement_factor"
-            )
-
-    @properties.observer("refinement_distance")
-    def _refinement_distance_validator(self, change):
-        if (
-            self.refinement_factor is not None
-            and len(change["value"]) != self.refinement_factor
-        ):
-            print("Number of refinement radii current DOES NOT match refinement_factor")
-
-    @properties.validator("indActive")
-    def _indActive_validator(self, change):
-
-        if len(change["value"]) != self.mesh.nC:
+    @BaseSimulation.mesh.setter
+    def mesh(self, value):
+        value = validate_type(
+            "mesh", value, (discretize.TensorMesh, discretize.TreeMesh), cast=False
+        )
+        if value.dim != 3:
             raise ValueError(
-                "Length of active topo cells array must equal number of mesh cells (nC = {})".format(
-                    self.mesh.nC
-                )
+                f"Mesh must be 3D tensor or 3D tree. Current mesh is {value.dim}"
             )
+        self._mesh = value
+
+    @property
+    def survey(self):
+        """The VRM survey.
+
+        Returns
+        -------
+        SimPEG.electromagnetics.viscous_temanent_magnetization.survey.Survey
+        """
+        if self._survey is None:
+            raise AttributeError("Simulation must have a survey.")
+        return self._survey
+
+    @survey.setter
+    def survey(self, value):
+        if value is not None:
+            value = validate_type("survey", value, SurveyVRM, cast=False)
+        self._survey = value
+
+    @property
+    def refinement_factor(self):
+        """The number of refinement distances.
+
+        Returns
+        -------
+        int
+        """
+        return len(self.refinement_distance)
+
+    @property
+    def refinement_distance(self):
+        """Sensitivity refinement radii from sources.
+
+        Returns
+        -------
+        numpy.ndarray
+        """
+        return self._refinement_distance
+
+    @refinement_distance.setter
+    def refinement_distance(self, value):
+        self._refinement_distance = validate_ndarray_with_shape(
+            "refinement_distance", value, shape=("*",)
+        )
+
+    @property
+    def indActive(self):
+        """Topography active cells.
+
+        Returns
+        -------
+        (mesh.n_cells) numpy.ndarray of bool
+        """
+        return self._indActive
+
+    @indActive.setter
+    def indActive(self, value):
+        self._indActive = validate_active_indices("indActive", value, self.mesh.n_cells)
 
     def _getH0matrix(self, xyz, pp):
 
@@ -408,10 +411,10 @@ class BaseVRMSimulation(BaseSimulation):
                         0.129485,
                     ],
                 ]
-                wt = wt[rxObj.quadOrder - 1]
+                wt = wt[rxObj.quadrature_order - 1]
                 nw = len(wt)
                 wt = (
-                    rxObj.nTurns
+                    rxObj.n_turns
                     * (rxObj.width / 2) ** 2
                     * np.reshape(np.outer(wt, wt), (1, nw ** 2))
                 )
@@ -441,20 +444,21 @@ class BaseVRMSimulation(BaseSimulation):
                     0.5
                     * rxObj.width
                     * np.reshape(
-                        np.kron(ds[rxObj.quadOrder - 1], np.ones(nw)), (nw ** 2, 1)
+                        np.kron(ds[rxObj.quadrature_order - 1], np.ones(nw)),
+                        (nw ** 2, 1),
                     )
                 )
                 s2 = (
                     0.5
                     * rxObj.width
                     * np.reshape(
-                        np.kron(np.ones(nw), ds[rxObj.quadOrder - 1]), (nw ** 2, 1)
+                        np.kron(np.ones(nw), ds[rxObj.quadrature_order - 1]),
+                        (nw ** 2, 1),
                     )
                 )
 
                 if dComp.lower() == "x":
                     for rr in range(0, nLoc):
-
                         u1 = np.kron(np.ones((nw ** 2, 1)), locs[rr, 0] - ax)
                         u1[np.abs(u1) < tol] = np.min(xyzh[:, 0]) / tol2
                         u2 = np.kron(np.ones((nw ** 2, 1)), locs[rr, 0] - bx)
@@ -531,7 +535,6 @@ class BaseVRMSimulation(BaseSimulation):
 
                 elif dComp.lower() == "y":
                     for rr in range(0, nLoc):
-
                         u1 = np.kron(np.ones((nw ** 2, 1)), locs[rr, 0] - ax) + np.kron(
                             s1, np.ones((1, nC))
                         )
@@ -608,7 +611,6 @@ class BaseVRMSimulation(BaseSimulation):
 
                 elif dComp.lower() == "z":
                     for rr in range(0, nLoc):
-
                         u1 = np.kron(np.ones((nw ** 2, 1)), locs[rr, 0] - ax) + np.kron(
                             s1, np.ones((1, nC))
                         )
@@ -805,7 +807,6 @@ class BaseVRMSimulation(BaseSimulation):
 
 
 class Simulation3DLinear(BaseVRMSimulation):
-
     """"""
 
     _A = None
@@ -813,15 +814,15 @@ class Simulation3DLinear(BaseVRMSimulation):
     _TisSet = False
     _xiMap = None
 
-    survey = properties.Instance("VRM Survey", SurveyVRM)
-
     xi, xiMap, xiDeriv = props.Invertible(
         "Amalgamated Viscous Remanent Magnetization Parameter xi = dchi/ln(tau2/tau1)"
     )
 
-    def __init__(self, mesh, **kwargs):
+    def __init__(self, mesh, xi=None, xiMap=None, **kwargs):
 
-        super(Simulation3DLinear, self).__init__(mesh, **kwargs)
+        super().__init__(mesh, **kwargs)
+        self.xi = xi
+        self.xiMap = xiMap
 
         nAct = list(self.indActive).count(True)
         if self.xiMap is None:
@@ -888,12 +889,11 @@ class Simulation3DLinear(BaseVRMSimulation):
                 waveObj = source_list[pp].waveform
 
                 for qq in range(0, nRx):
-
                     times = receiver_list[qq].times
                     nLoc = np.shape(receiver_list[qq].locations)[0]
 
                     I = sp.diags(np.ones(nLoc))
-                    eta = waveObj.getCharDecay(receiver_list[qq].fieldType, times)
+                    eta = waveObj.getCharDecay(receiver_list[qq].field_type, times)
                     eta = np.atleast_2d(eta).T
 
                     T.append(sp.kron(I, eta))
@@ -969,7 +969,6 @@ class Simulation3DLinear(BaseVRMSimulation):
 
 
 class Simulation3DLogUniform(BaseVRMSimulation):
-
     """"""
 
     _A = None
@@ -982,9 +981,14 @@ class Simulation3DLogUniform(BaseVRMSimulation):
     tau1 = props.PhysicalProperty("Low bound time-relaxation constant")
     tau2 = props.PhysicalProperty("Upper bound time-relaxation constant")
 
-    def __init__(self, mesh, **kwargs):
-
-        super(Simulation3DLogUniform, self).__init__(mesh, **kwargs)
+    def __init__(
+        self, mesh, survey=None, chi0=None, dchi=None, tau1=None, tau2=None, **kwargs
+    ):
+        super(Simulation3DLogUniform, self).__init__(mesh=mesh, survey=survey, **kwargs)
+        self.chi0 = chi0
+        self.dchi = dchi
+        self.tau1 = tau1
+        self.tau2 = tau2
 
     @property
     def A(self):
@@ -1035,10 +1039,9 @@ class Simulation3DLogUniform(BaseVRMSimulation):
             waveObj = source_list[pp].waveform
 
             for qq in range(0, nRx):
-
                 times = receiver_list[qq].times
                 eta = waveObj.getLogUniformDecay(
-                    receiver_list[qq].fieldType,
+                    receiver_list[qq].field_type,
                     times,
                     self.chi0,
                     self.dchi,
@@ -1049,18 +1052,3 @@ class Simulation3DLogUniform(BaseVRMSimulation):
                 f.append(mkvc(self.A[qq] @ eta))
 
         return np.array(np.hstack(f))
-
-
-############
-# Deprecated
-############
-
-
-@deprecate_class(removal_version="0.16.0", error=True)
-class Problem_Linear(Simulation3DLinear):
-    pass
-
-
-@deprecate_class(removal_version="0.16.0", error=True)
-class Problem_LogUnifrom(Simulation3DLogUniform):
-    pass
