@@ -1,67 +1,170 @@
-from __future__ import print_function
-
-import properties
 import numpy as np
 import scipy.sparse as sp
 import gc
 from .data_misfit import BaseDataMisfit
-from .props import BaseSimPEG, Model
-from .regularization import BaseRegularization, BaseComboRegularization, Sparse
+from .regularization import BaseRegularization, WeightedLeastSquares, Sparse
 from .objective_function import BaseObjectiveFunction, ComboObjectiveFunction
-from .utils import callHooks, timeIt
+from .optimization import Minimize
+from .utils import (
+    callHooks,
+    timeIt,
+    Counter,
+    validate_float,
+    validate_type,
+    validate_ndarray_with_shape,
+)
+from .simulation import DefaultSolver
 
 
-class BaseInvProblem(BaseSimPEG):
+class BaseInvProblem:
     """BaseInvProblem(dmisfit, reg, opt)"""
 
-    #: Trade-off parameter
-    beta = 1.0
-
-    #: Print debugging information
-    debug = False
-
-    #: Set this to a SimPEG.utils.Counter() if you want to count things
-    counter = None
-
-    #: DataMisfit
-    dmisfit = None
-
-    #: Regularization
-    reg = None
-
-    #: Optimization program
-    opt = None
-
-    #: List of strings, e.g. ['_MeSigma', '_MeSigmaI']
-    deleteTheseOnModelUpdate = []
-
-    model = Model("Inversion model.")
-
-    @properties.observer("model")
-    def _on_model_update(self, value):
-        """
-        Sets the current model, and removes dependent properties
-        """
-        for prop in self.deleteTheseOnModelUpdate:
-            if hasattr(self, prop):
-                delattr(self, prop)
-
-    def __init__(self, dmisfit, reg, opt, **kwargs):
-        super(BaseInvProblem, self).__init__(**kwargs)
-        assert isinstance(dmisfit, BaseDataMisfit) or isinstance(
-            dmisfit, BaseObjectiveFunction
-        ), "dmisfit must be a DataMisfit or ObjectiveFunction class."
+    def __init__(
+        self, dmisfit, reg, opt, beta=1.0, debug=False, counter=None, **kwargs
+    ):
+        super().__init__(**kwargs)
         assert isinstance(reg, BaseRegularization) or isinstance(
             reg, BaseObjectiveFunction
         ), "reg must be a Regularization or Objective Function class."
+
         self.dmisfit = dmisfit
+
         self.reg = reg
         self.opt = opt
+        self.beta = beta
+        self.debug = debug
+        self.counter = counter
+        self.model = None
         # TODO: Remove: (and make iteration printers better!)
         self.opt.parent = self
         self.reg.parent = self
         self.dmisfit.parent = self
 
+    #: Trade-off parameter
+    @property
+    def beta(self):
+        """Trade-off parameter
+
+        Returns
+        -------
+        float
+        """
+        return self._beta
+
+    @beta.setter
+    def beta(self, value):
+        self._beta = validate_float("beta", value, min_val=0.0)
+
+    @property
+    def debug(self):
+        """Whether to print debugging information.
+
+        Returns
+        -------
+        bool
+        """
+        return self._debug
+
+    @debug.setter
+    def debug(self, value):
+        self._debug = validate_type("debug", value, bool)
+
+    @property
+    def counter(self):
+        """Set this to a `SimPEG.utils.Counter` if you want to count things.
+
+        Returns
+        -------
+        None or SimPEG.utils.Counter
+        """
+        return self._counter
+
+    @counter.setter
+    def counter(self, value):
+        if value is not None:
+            value = validate_type("counter", value, Counter, cast=False)
+        self._counter = value
+
+    @property
+    def dmisfit(self):
+        """The data misfit.
+
+        Returns
+        -------
+        SimPEG.objective_function.ComboObjectiveFunction
+        """
+        return self._dmisfit
+
+    @dmisfit.setter
+    def dmisfit(self, value):
+        value = validate_type("dmisfit", value, BaseObjectiveFunction, cast=False)
+        if not isinstance(value, ComboObjectiveFunction):
+            value = ComboObjectiveFunction(objfcts=[value])
+        self._dmisfit = value
+
+    @property
+    def reg(self):
+        """The regularization object for the inversion
+
+        Returns
+        -------
+        SimPEG.objective_function.ComboObjectiveFunction
+        """
+        return self._reg
+
+    @reg.setter
+    def reg(self, value):
+        value = validate_type("reg", value, BaseObjectiveFunction, cast=False)
+        if not isinstance(value, ComboObjectiveFunction):
+            value = ComboObjectiveFunction(objfcts=[value])
+        self._reg = value
+
+    @property
+    def opt(self):
+        """The optimization routine.
+
+        Returns
+        -------
+        SimPEG.optimization.Minimize
+        """
+        return self._opt
+
+    @opt.setter
+    def opt(self, value):
+        self._opt = validate_type("opt", value, Minimize, cast=False)
+
+    @property
+    def deleteTheseOnModelUpdate(self):
+        """A list of properties stored on this object to delete when the model is updated
+
+        Returns
+        -------
+        list of str
+            For example `['_MeSigma', '_MeSigmaI']`.
+        """
+        return []
+
+    @property
+    def model(self):
+        """The inversion model.
+
+        Returns
+        -------
+        numpy.ndarray
+        """
+        return self._model
+
+    @model.setter
+    def model(self, value):
+        if value is not None:
+            value = validate_ndarray_with_shape(
+                "model", value, shape=[("*",), ("*", "*")], dtype=None
+            )
+        for prop in self.deleteTheseOnModelUpdate:
+            if hasattr(self, prop):
+                delattr(self, prop)
+        self._model = value
+
     @callHooks("startup")
     def startup(self, m0):
         """startup(m0)
@@ -71,48 +174,56 @@ class BaseInvProblem(BaseSimPEG):
         if self.debug:
             print("Calling InvProblem.startup")
 
-        if hasattr(self.reg, "mref") and getattr(self.reg, "mref", None) is None:
-            print("SimPEG.InvProblem will set Regularization.mref to m0.")
-            self.reg.mref = m0
-
-        if isinstance(self.reg, ComboObjectiveFunction) and not isinstance(
-            self.reg, BaseComboRegularization
-        ):
-            for fct in self.reg.objfcts:
-                if hasattr(fct, "mref") and getattr(fct, "mref", None) is None:
-                    print("SimPEG.InvProblem will set Regularization.mref to m0.")
-                    fct.mref = m0
+        for fct in self.reg.objfcts:
+            if (
+                hasattr(fct, "reference_model")
+                and getattr(fct, "reference_model", None) is None
+            ):
+                print(
+                    "SimPEG.InvProblem will set Regularization.reference_model to m0."
+                )
+                fct.reference_model = m0
 
         self.phi_d = np.nan
         self.phi_m = np.nan
 
         self.model = m0
 
-        if isinstance(self.dmisfit, BaseDataMisfit):
-            if getattr(self.dmisfit.simulation, "solver", None) is not None:
+        solver = DefaultSolver
+        set_default = True
+        for objfct in self.dmisfit.objfcts:
+
+            if (
+                isinstance(objfct, BaseDataMisfit)
+                and getattr(objfct.simulation, "solver", None) is not None
+            ):
+                solver = objfct.simulation.solver
+                solver_opts = objfct.simulation.solver_opts
                 print(
                     """
-        SimPEG.InvProblem is setting bfgsH0 to the inverse of the eval2Deriv.
-        ***Done using same Solver and solverOpts as the problem***"""
+                        SimPEG.InvProblem is setting bfgsH0 to the inverse of the eval2Deriv.
+                        ***Done using same Solver, and solver_opts as the {} problem***
+                        """.format(
+                        objfct.simulation.__class__.__name__
+                    )
                 )
-                self.opt.bfgsH0 = self.dmisfit.simulation.solver(
-                    self.reg.deriv2(self.model), **self.dmisfit.simulation.solver_opts
+                set_default = False
+                break
+        if set_default:
+            print(
+                """
+                    SimPEG.InvProblem is setting bfgsH0 to the inverse of the eval2Deriv.
+                    ***Done using the default solver {} and no solver_opts.***
+                    """.format(
+                    DefaultSolver.__name__
                 )
-        elif isinstance(self.dmisfit, BaseObjectiveFunction):
-            for objfct in self.dmisfit.objfcts:
-                if isinstance(objfct, BaseDataMisfit):
-                    if getattr(objfct.simulation, "solver", None) is not None:
-                        print(
-                            """
-        SimPEG.InvProblem is setting bfgsH0 to the inverse of the eval2Deriv.
-        ***Done using same Solver and solver_opts as the {} problem***""".format(
-                                objfct.simulation.__class__.__name__
-                            )
-                        )
-                        self.opt.bfgsH0 = objfct.simulation.solver(
-                            self.reg.deriv2(self.model), **objfct.simulation.solver_opts
-                        )
-                        break
+            )
+            solver = DefaultSolver
+            solver_opts = {}
+
+        self.opt.bfgsH0 = solver(
+            sp.csr_matrix(self.reg.deriv2(self.model)), **solver_opts
+        )
 
     @property
     def warmstart(self):
@@ -161,16 +272,13 @@ class BaseInvProblem(BaseSimPEG):
         return f
 
     def get_dpred(self, m, f):
-        if isinstance(self.dmisfit, BaseDataMisfit):
-            return self.dmisfit.simulation.dpred(m, f=f)
-        elif isinstance(self.dmisfit, BaseObjectiveFunction):
-            dpred = []
-            for i, objfct in enumerate(self.dmisfit.objfcts):
-                if hasattr(objfct, "survey"):
-                    dpred += [objfct.survey.dpred(m, f=f[i])]
-                else:
-                    dpred += []
-            return dpred
+        dpred = []
+        for i, objfct in enumerate(self.dmisfit.objfcts):
+            if hasattr(objfct, "simulation"):
+                dpred += [objfct.simulation.dpred(m, f=f[i])]
+            else:
+                dpred += []
+        return np.hstack(dpred)
 
     @timeIt
     def evalFunction(self, m, return_g=True, return_H=True):
@@ -191,7 +299,7 @@ class BaseInvProblem(BaseSimPEG):
         self.phi_d, self.phi_d_last = phi_d, self.phi_d
         self.phi_m, self.phi_m_last = phi_m, self.phi_m
 
-        # Only works for Tikhonov
+        # Only works for WeightedLeastSquares regularization
         if self.opt.print_type == "ubc":
 
             self.phi_s = 0.0
@@ -199,7 +307,7 @@ class BaseInvProblem(BaseSimPEG):
             self.phi_y = 0.0
             self.phi_z = 0.0
 
-            if not isinstance(self.reg, BaseComboRegularization):
+            if not isinstance(self.reg, WeightedLeastSquares):
                 regs = self.reg.objfcts
                 mults = self.reg.multipliers
             else:
@@ -210,7 +318,7 @@ class BaseInvProblem(BaseSimPEG):
                     i_s, i_x, i_y, i_z = 0, 1, 2, 3
                 else:
                     i_s, i_x, i_y, i_z = 0, 1, 3, 5
-                dim = reg.regmesh.dim
+                dim = reg.regularization_mesh.dim
                 self.phi_s += mult * reg.objfcts[i_s](m) * reg.alpha_s
                 self.phi_x += mult * reg.objfcts[i_x](m) * reg.alpha_x
                 if dim > 1:
