@@ -1,6 +1,7 @@
 import re
 import os
 from discretize import TensorMesh
+from discretize.utils import active_from_xyz
 
 try:
     from SimPEG import utils
@@ -168,7 +169,7 @@ class GravityDriver_Inv(object):
     @property
     def mesh(self):
         if getattr(self, "_mesh", None) is None:
-            self._mesh = TensorMesh.readUBC(self.basePath + self.mshfile)
+            self._mesh = TensorMesh.read_UBC(self.basePath + self.mshfile)
         return self._mesh
 
     @property
@@ -187,7 +188,7 @@ class GravityDriver_Inv(object):
             if getattr(self, "topofile", None) is not None:
                 topo = np.genfromtxt(self.basePath + self.topofile, skip_header=1)
                 # Find the active cells
-                active = utils.surface2ind_topo(self.mesh, topo, "N")
+                active = active_from_xyz(self.mesh, topo, "N")
 
             elif isinstance(self._staticInput, float):
                 active = self.m0 != self._staticInput
@@ -196,8 +197,7 @@ class GravityDriver_Inv(object):
                 # Read from file active cells with 0:air, 1:dynamic, -1 static
                 active = self.activeModel != 0
 
-            inds = np.where(active)[0]
-            self._activeCells = inds
+            self._activeCells = active
 
             # Reduce m0 to active space
             if len(self.m0) > len(self._activeCells):
@@ -243,7 +243,7 @@ class GravityDriver_Inv(object):
                 self._m0 = np.ones(self.nC) * self.mstart
             else:
 
-                self._m0 = TensorMesh.readModelUBC(
+                self._m0 = TensorMesh.read_model_UBC(
                     self.mesh, self.basePath + self.mstart
                 )
 
@@ -255,7 +255,7 @@ class GravityDriver_Inv(object):
             if isinstance(self._mrefInput, float):
                 self._mref = np.ones(self.nC) * self._mrefInput
             else:
-                self._mref = TensorMesh.readModelUBC(
+                self._mref = TensorMesh.read_model_UBC(
                     self.mesh, self.basePath + self._mrefInput
                 )
                 self._mref = self._mref[self.activeCells]
@@ -266,7 +266,7 @@ class GravityDriver_Inv(object):
         if getattr(self, "_activeModel", None) is None:
             if isinstance(self._staticInput, str):
                 # Read from file active cells with 0:air, 1:dynamic, -1 static
-                self._activeModel = TensorMesh.readModelUBC(
+                self._activeModel = TensorMesh.read_model_UBC(
                     self.mesh, self.basePath + self._staticInput
                 )
 
