@@ -1,27 +1,22 @@
-from __future__ import print_function
 import numpy as np
 
 
 def read_GOCAD_ts(tsfile):
-    """
+    r"""Read GOCAD triangulated surface (*.ts) file
 
-    Read GOCAD triangulated surface (*.ts) file
-    INPUT:
-    tsfile: Triangulated surface
-
-    OUTPUT:
-    vrts : Array of vertices in XYZ coordinates [n x 3]
-    trgl : Array of index for triangles [m x 3]. The order of the vertices
-            is important and describes the normal
-            n = cross( (P2 - P1 ) , (P3 - P1) )
-
-    Author: @fourndo
-
-
-    .. note::
-
-        Remove all attributes from the GoCAD surface before exporting it!
+    Parameters
+    ----------
+    tsfile : str
+        Path to triangulated surface (*.ts) file
 
+    Returns
+    -------
+    vrts : (n, 3) numpy.ndarray
+        Vertices in XYZ coordinates
+    trgl : (m, 3) numpy.ndarray of int
+        Array of indexes where each row represents the indexes for a particular
+        triangle. Note that the order of the vertices matter, as they defined
+        normal vectors; i.e. :math:`\hat{n} = (\mathbf{p_2 - p_1}) \times (\mathbf{p_3 - p_1})`
     """
 
     import re
@@ -68,87 +63,23 @@ def read_GOCAD_ts(tsfile):
     return vrtx, trgl
 
 
-def surface2inds(vrtx, trgl, mesh, boundaries=True, internal=True):
-    """ "
-    Function to read gocad polystructure file and output indexes of
-    mesh within the structure.
-
-    """
-    import vtk
-    import vtk.util.numpy_support as npsup
-
-    # Adjust the index
-    trgl = trgl - 1
-
-    # Make vtk pts
-    ptsvtk = vtk.vtkPoints()
-    ptsvtk.SetData(npsup.numpy_to_vtk(vrtx, deep=1))
-
-    # Make the polygon connection
-    polys = vtk.vtkCellArray()
-    for face in trgl:
-        poly = vtk.vtkPolygon()
-        poly.GetPointIds().SetNumberOfIds(len(face))
-        for nrv, vert in enumerate(face):
-            poly.GetPointIds().SetId(nrv, vert)
-        polys.InsertNextCell(poly)
-
-    # Make the polydata, structure of connections and vrtx
-    polyData = vtk.vtkPolyData()
-    polyData.SetPoints(ptsvtk)
-    polyData.SetPolys(polys)
-
-    # Make implicit func
-    ImpDistFunc = vtk.vtkImplicitPolyDataDistance()
-    ImpDistFunc.SetInput(polyData)
-
-    # Convert the mesh
-    vtkMesh = vtk.vtkRectilinearGrid()
-    vtkMesh.SetDimensions(mesh.nNx, mesh.nNy, mesh.nNz)
-    vtkMesh.SetXCoordinates(npsup.numpy_to_vtk(mesh.vectorNx, deep=1))
-    vtkMesh.SetYCoordinates(npsup.numpy_to_vtk(mesh.vectorNy, deep=1))
-    vtkMesh.SetZCoordinates(npsup.numpy_to_vtk(mesh.vectorNz, deep=1))
-    # Add indexes
-    vtkInd = npsup.numpy_to_vtk(np.arange(mesh.nC), deep=1)
-    vtkInd.SetName("Index")
-    vtkMesh.GetCellData().AddArray(vtkInd)
-
-    extractImpDistRectGridFilt = vtk.vtkExtractGeometry()  # Object constructor
-    extractImpDistRectGridFilt.SetImplicitFunction(ImpDistFunc)  #
-    extractImpDistRectGridFilt.SetInputData(vtkMesh)
-
-    if boundaries is True:
-        extractImpDistRectGridFilt.ExtractBoundaryCellsOn()
-
-    else:
-        extractImpDistRectGridFilt.ExtractBoundaryCellsOff()
-
-    if internal is True:
-        extractImpDistRectGridFilt.ExtractInsideOn()
-
-    else:
-        extractImpDistRectGridFilt.ExtractInsideOff()
-
-    print("Extracting indices from grid...")
-    # Executing the pipe
-    extractImpDistRectGridFilt.Update()
-
-    # Get index inside
-    insideGrid = extractImpDistRectGridFilt.GetOutput()
-    insideGrid = npsup.vtk_to_numpy(insideGrid.GetCellData().GetArray("Index"))
-
-    # Return the indexes inside
-    return insideGrid
-
-
 def download(url, folder=".", overwrite=False, verbose=True):
-    """
-    Function to download all files stored in a cloud directory
+    """Download all files stored in a cloud directory.
 
-    :param str url: url or list of urls for the file(s) to be downloaded ("https://...")
-    :param str folder: folder to where the directory is created and files downloaded (default is the current directory)
-    :param bool overwrite: overwrite if a file with the specified name already exists
-    :param bool verbose: print out progress
+    Parameters
+    ----------
+    url : str or list of str
+        A single URL or a list of URLs for all cloud directories containing files
+        you wish to download.
+    folder : str
+        Path to the directory where all files are downloaded to. This function will
+        create the directory if the directory does not already exist.
+    overwrite : bool
+        If ``True``, the function will overwrite preexisting files with new files
+        in the case they both have identical names. If ``False``, the name of the
+        file being downloaded is change if it is identical to a preexisting file name.
+    verbose : bool
+        Print download progress
     """
 
     # Download from cloud
